import re
from typing import List, NewType, Union

Move = NewType('Move', Union[str, List])


def _get_pgn_list(path: str) -> list:
    with open(file=path, mode='r') as f:
        lines = f.read().splitlines()
        return lines


def _get_tags(pgn: list) -> dict:
    tag_list = [tag for tag in pgn if re.search(r'^\[', tag)]
    tag_dict: dict = {}
    for tag in tag_list:
        key = tag.split(" ", 1)[0][1:]
        val = tag.split(" ", 1)[1][1:-2]
        tag_dict[key] = val

    return tag_dict


def _get_moves(pgn: list) -> List[Move]:
    for line in pgn:
        if re.search(r'^1\. ', line):
            movetext: str = line

    movetext_items = movetext.split(" ")
    moves = []
    for i, item in enumerate(movetext_items):
        if re.search(r'\d\.', item):
            move = [movetext_items[i],
                    movetext_items[i + 1],
                    movetext_items[i + 2]]
            moves.append(move)

    return moves



from typing import List
from pypgn.game_utils import (
    _get_tags, _get_pgn_list, _get_moves, Move)


class Game:
    """A class used to represent a chess game round.

    ...

    Attributes
    ----------
    pgn: list
        a list of lines of the original PGN file
    tags: Mapping
        a map of the parsed PGN file game tags
    moves: List[Move]
        a list of Moves, with [0] - move number,
                              [1] - white ply,
                              [2] - black ply

    Methods
    -------
    get_pgn_list()
        Returns a list of lines of the PGN file
    get_tags()
        Returns a map of tags of the PGN file
    get_moves()
        Returns a list of moves of the movetext
    get_tag(name: str)
        Returns a value for a given key in the parsed PGN tags
    get_move(index: int)
        Returns a move list for a given move number
    get_ply(index: int, player: str)
        Returns a ply for a given move for a given player
    get_move_count()
        Returns the total move count for the given game
    get_result()
        Returns the result of the game
    get_date()
        Returns the date of the played game
    get_move_range(start: int, end: int)
        Returns a list of moves from given start to end index
    """

    def __init__(self, file_path: str):
        """

        :param file_path: path to pgn file
        :type file_path: str
        """
        self.pgn: list = _get_pgn_list(file_path)
        self.tags: dict = _get_tags(self.pgn)
        self.moves: List[Move] = _get_moves(self.pgn)

    def get_pgn_list(self) -> list:
        """Gets and returns a list of lines of the PGN

        :return: a list of lines of the parsed PGN
        :rtype: list
        """
        return self.pgn

    def get_tag(self, name: str) -> str:
        """Gets and returns a tag for a given key name

        :param name: Key name
        :type name: str
        :return: Value of a tag
        :rtype: str
        """
        return self.tags[name]

    def get_tags(self) -> dict:
        """Gets and returns a map of metadata tags of the PGN

        :return: Map of PGN tags
        :rtype: dict
        """
        return self.tags

    def get_move(self, index: int) -> Move:
        """Gets and returns a move of a certain number

        :param index: Move number
        :type index: int
        :return: Move
        :rtype: Move
        """
        return self.moves[index - 1]

    def get_moves(self) -> List[Move]:
        """Gets and returns a list of moves

        :return: A list of Moves
        :rtype: List[Move]
        """
        return self.moves

    def get_ply(self, index: int, player: str) -> str:
        """Gets and returns a ply for a given move

        :param index: Move number
        :type index: int
        :param player: Player color (white or black)
        :type player: str
        :return: Ply of player of given color
        :rtype: str
        """
        return self.moves[index - 1][1 if 'w' in player.lower() else 2]

    def get_move_count(self) -> int:
        """Gets and returns the total number of moves in the game

        :return: Total number of moves
        :rtype: int
        """
        return len(self.moves)

    def get_result(self) -> str:
        """Gets and returns the game result

        :return: Result of the game
        :rtype: str
        """
        return self.get_tag('Result')

    def get_date(self) -> str:
        """Gets and returns the date of the game

        :return: Date of the game in format YYYY.MM.DD
        """
        return self.get_tag('Date')

    def get_move_range(self, start: int, end: int) -> List[Move]:
        """Gets and returns a range of moves

        :param start: Start index of moves to get
        :param end: End index of moves to get
        :return: List of moves in given range
        """
        return self.moves[start - 1:end]



from setuptools import setup

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='pypgn',
    long_description=long_description,
    long_description_content_type='text/markdown',
    version='0.3.9',
    packages=['pypgn'],
    url='https://github.com/DaniruKun/pypgn',
    license='MPL-2.0',
    author='Daniils Petrovs',
    author_email='thedanpetrov@gmail.com',
    keywords=['pgn', 'chess', 'game', 'notation'],
    description='Library for parsing and manipulating PGN format files.',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Text Processing',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
      ],
)



import pytest
import os


@pytest.fixture(scope="module")
def game_obj():
    from pypgn.game import Game
    from pathlib import Path
    pgn_file_path = Path('test/resources') if 'test' not in os.getcwd() else Path('resources')

    return Game(str(pgn_file_path / 'test.pgn'))


class TestGame:
    def test_get_pgn_list(self, game_obj):
        assert len(game_obj.get_pgn_list()) == 20

    def test_get_tags(self, game_obj):
        assert type(game_obj.get_tags()) == dict

    def test_get_tag_value(self, game_obj):
        assert game_obj.get_tag('Event') == "Rated Blitz game"
        assert game_obj.get_tag('Site') == "https://lichess.org/#"
        assert game_obj.get_tag('UTCDate') == "2019.11.06"

    def test_get_move(self, game_obj):
        assert game_obj.get_move(4) == ["4.", "d4", "Nc6"]

    def test_get_moves(self, game_obj):
        moves = game_obj.get_moves()
        assert type(moves) == list
        assert moves[0] == ["1.", "e4", "c6"]

    def test_get_ply(self, game_obj):
        for i in range(1, game_obj.get_move_count()):
            move = game_obj.get_move(i)
            assert move[1] == game_obj.get_ply(i, 'w') \
                   and move[2] == game_obj.get_ply(i, 'b'), \
                "Non-matching ply pair at move %s !" % str(i)

    def test_get_move_count(self, game_obj):
        assert game_obj.get_move_count() == len(game_obj.get_moves())

    def test_get_result(self, game_obj):
        assert game_obj.get_result() == game_obj.get_tag('Result')

    def test_get_date(self, game_obj):
        assert game_obj.get_date() == game_obj.get_tag('UTCDate')

    def test_get_move_range(self, game_obj):
        assert game_obj.get_move_range(1, 28) == game_obj.get_moves()
